	; make all RGB pixels flash fast, supposed to wake
	; up some stuck sub pixels on TFT. do not look at
	; the screen unless you want to hurt your brain :-p.
	;
	; Flash scheme suggested by Mateusz Viste:
	; do
	;   R 15-70 msec random, then black 25 msec
	;   G 15-70 msec random, then black 25 msec
	;   B 15-70 msec random, then black 25 msec
	; repeat
	; At usual refresh rates, 1 frame = 10 to 20 msec
	;
	; http://sourceforge.net/projects/stucksweep uses:
	; pick random color from R, G, B, white, black each 15 msec
	;
	; UDPIX recommends 25 msec delay, lets you config flashes?
	;
	; Your brain will hurt less if you flash only the pixels
	; in the area of the damaged pixel, but this will make
	; the program more complex...

%define red 4
%define green 2
%define blue 1
%define cyan (blue+green)
%define magenta (red+blue)
%define yellow (red+green)
%define white (red+green+blue)
%define black 0

	org 100h
start:	push ds
	xor ax,ax
	mov ds,ax
	mov dx,[ds:463h]	; graphics I/O base
	pop ds
	add dx,6		; status port
	mov [port],dx

	mov ax,3		; select 80x25 text
	int 10h

	cld
	mov si,81h
cmd:	lodsb
	cmp al,' '
	jz cmd
	cmp al,13
	jz help
	cmp al,'?'
	jz help
	cmp al,'/'
	jz cmd
	cmp al,'-'
	jz cmd
	or al,20h	; make lower case
	cmp al,'h'
	jz help
	cmp al,'r'
	jz redflash
	cmp al,'g'
	jz greenflash
	cmp al,'b'
	jz blueflash
	cmp al,'k'
	jz killflash	; killdeadpixel.com style
	cmp al,'m'
	jnz nom
	jmp matflash	; Mateusz Viste style
nom:	cmp al,'s'
	jnz help
	jmp sweepflash	; stucksweep.sf.net style

help:	mov dx,helpmsg
	mov ah,9
	int 21h
	mov ax,4c01h
	int 21h

redflash:
	mov al,red
	jmp flashy
greenflash:
	mov al,green
	jmp flashy
blueflash:
	mov al,blue
	jmp flashy

flashy:	push ax
	call setrgb
	call waitretrace
	mov al,black
	call setrgb
	call waitretrace
	call keytest
	pop ax
	jmp short flashy

killflash:	; note that killdeadpixel.com is better: a small size
		; animated gif where adjacent pixels are in different
		; phase of the cycle, looks greyish, hurts less brains.
	mov al,magenta
	call setrgb
	call waitretrace
	mov al,white
	call setrgb
	call waitretrace
	mov al,blue
	call setrgb
	call waitretrace
	mov al,red
	call setrgb
	call waitretrace
	mov al,yellow
	call setrgb
	call waitretrace
	mov al,cyan
	call setrgb
	call waitretrace
	mov al,green
	call setrgb
	call waitretrace
	mov al,black
	call setrgb
	call waitretrace
	call keytest
	jmp short killflash

matflash:
	mov al,red
	call setrgb
	call waitrnd
	mov al,black
	call setrgb
	call waitrnd
	mov al,green
	call setrgb
	call waitrnd
	mov al,black
	call setrgb
	call waitrnd
	mov al,blue
	call setrgb
	call waitrnd
	mov al,black
	call setrgb
	call waitrnd
	call keytest
	jmp short matflash

waitrnd:	; random wait, 1 to 6 retraces, changes ax
	call waitretrace
	call random
	and ax,1023
	push bx
	mov bx,6
	div bl	; ah = 0..5, al = 0..170
	mov bl,ah
wx:	or bl,bl
	jz wdone
	call waitretrace
	dec bl
	jmp short wx
wdone:	pop bx
	ret

sweepflash:
	call random
	and ax,1023
	push bx
	mov bx,5
	div bl	; ah = 0..4, al = 0..204
	mov bl,ah
	mov bh,0
	mov al,[sweeps+bx]
	pop bx
	call setrgb
	call waitretrace
	call keytest
	jmp short sweepflash

sweeps	db red, green, blue, black, white	; stucksweep.sf.net list

setrgb:		; set color selected on 3 bit define in AL
	push ax
	push bx
	push cx
	push dx
	xor bx,bx	; color 0
	xor cx,cx
	xor dx,dx
	test al,red
	jz nored
	mov dh,63	; dl is unused
nored:	test al,green
	jz nogrn
	mov ch,63
nogrn:	test al,blue
	jz noblue
	mov cl,63
noblue:	mov ax,1010h	; set VGA color	(1000h is EGA color bl=bh)
	int 10h
	pop dx
	pop cx
	pop bx
	pop ax
	ret

keytest:	; exit if any key pressed
	push ax
	mov ah,1
	int 16h
	jnz keyhit
	pop ax
	ret
keyhit:	pop ax
	pop ax	; return addr
	mov ax,3	; text mode again
	int 10h
	mov ax,4c00h
	int 21h

waitretrace:	; during retrace, port test 8 is 8
	push ax
	push cx
	push dx
	mov dx,[port]
	push ds
	xor ax,ax
	mov ds,ax
	mov cx,[ds:6ch]		; timer count (18 Hz)
w1:	cmp cx,[ds:6ch]
	jnz wabort		; just in case the port is stuck ;-)
	in al,dx
	test al,8		; already in retrace?
	jnz w1
w2:	cmp cx,[ds:6ch]
	jnz wabort		; just in case the port is stuck ;-)
	in al,dx
	test al,8		; wait until retrace starts
	jz w2
wabort:	pop ds
	pop dx
	pop cx
	pop ax
	ret


; 386 compatible linear pseudo random number generator
random:	; return a random value in eax
	imul eax, [_seed], 41c64e6dh ; 9 bytes
	add eax, 3039h ; 6 bytes
 	and eax, 7fffffffh ; 6 bytes
 	mov [_seed], eax ; 4 bytes (doing byte AND on [_seed+3] would save 1 byte)
	ret

_seed	dd 0deadbeefh ; or init from 40:6c at program start

helpmsg	db "RGBFLASH Public Domain software by Eric Auer 2007:",13,10
	db "Displays flashy stuff which might wake up stuck TFT LCD pixels",13,10
	db "Please do not look at the flashing for longer time!",13,10
	db "It may help to (gently!) press the affected area, too.",13,10
	db "Use RGBFLASH at your own risk. Needs VGA and 386 or better.",13,10
	db "Hit any key to stop flashing.",13,10
	db 13,10
	db "Available flash patterns: Select by running 'RGBFLASH option'",13,10
	db "R flashes between red and black",13,10
	db "G flashes between green and black",13,10
	db "B flashes between blue and black",13,10
	db "K cycle through black, magenta, white, blue, red, yellow, cyan, green",13,10
	db "M cycle through red, black, green, black, blue, black, random delays",13,10
	db "S do random pick of red, green, blue, black, white each frame",13,10
	db 13,10
	db "Options k, m and s are inspired by killdeadpixel.com, Mateusz Viste",13,10
	db "and http://sourceforge.net/projects/stucksweep respectively :-)",13,10
	db "$"

port	dw 0	; graphics status port, 3bah or 3dah usually
